#!/usr/bin/env python
# simple broadcast FM receiver
# take in raw.pcm 8 bit unsigned file from cxcap (27 MHz sampling rate with FM IF at 10.7 MHz)
# output is audio.pcm (16 bit mono, little endian, 43200 Hz audio)
from gnuradio import *
from gnuradio import gr
from gnuradio import audio

def build_graph (IF_freq):

    input_rate = 27e6
    CFIR_decimate = 125
    RFIR_decimate = 5
    fm_demod_gain = 2200
    quad_rate = input_rate / CFIR_decimate
    audio_rate = quad_rate / RFIR_decimate
    volume = 1.0


    # compute FIR filter taps for channel selection
    channel_coeffs = \
      gr.firdes.low_pass (
        1.0,          # gain
        input_rate,   # sampling rate
        250e3,        # low pass cutoff freq
        8*100e3,      # width of trans. band
        gr.firdes.WIN_HAMMING)

    chan_filter = \
      gr.freq_xlating_fir_filter_fcf(CFIR_decimate,
                                 channel_coeffs,
                                 IF_freq,input_rate)

    fm_demod = \
      gr.quadrature_demod_cf (volume * fm_demod_gain)
    # compute FIR filter taps for audio filter
    width_of_transition_band = audio_rate / 32
    audio_coeffs = \
      gr.firdes.low_pass (
        1.0,            # gain
        quad_rate,      # sampling rate
        audio_rate/2 - width_of_transition_band,
        width_of_transition_band,
        gr.firdes.WIN_HAMMING)

    audio_filter = \
      gr.fir_filter_fff (RFIR_decimate, audio_coeffs)



    fg =  gr.flow_graph()

    src = gr.file_source(1,"./raw.pcm",0)

    remove_offset = gr.add_const_ff(-135);
    scale = gr.multiply_const_ff(300);
    convert2float = gr.uchar_to_float()

    float2short = gr.float_to_short()
    file_sink = gr.file_sink(2,"./audio.pcm")
    audio_sink =  audio.sink(int(audio_rate))


    fg.connect(src,convert2float)
    fg.connect(convert2float,remove_offset)
    fg.connect(remove_offset,scale)
    fg.connect(scale,chan_filter)
    fg.connect(chan_filter,fm_demod)
    fg.connect(fm_demod,audio_filter)

#   fg.connect(audio_filter,audio_sink)
#   if PC is fast enough maybe can sink direct to sound card
#   to try uncomment line above and comment out 2 lines below	

    fg.connect(audio_filter,float2short)
    fg.connect(float2short,file_sink)
  

    return fg
if __name__ == '__main__':

    IF_freq = 10.7e6 
    fg = build_graph (IF_freq)
    fg.start ()        # fork thread(s) and return
    raw_input ('Press Enter to quit: ')
    fg.stop ()
